-- 
-- Author: John Moskal
--  Created: 03-18-2005
-- Modified: 04-07-2005
--
-- Modifications/Comments:
-- 
-- 
------------------------------------------------------------------------------------------

library work;
library ieee;
use ieee.std_logic_1164.all;

entity HazDet is
port
(
   Clk:           in std_logic;
   Rst:           in std_logic;
   O,F:           in std_logic_vector(5 downto 0);   -- Opcode and Function
   D_RsAdr,
   D_RtAdr:       in std_logic_vector(4 downto 0);
   E_DstRegAdr:   in std_logic_vector(4 downto 0);
   M_DstRegAdr:   in std_logic_vector(4 downto 0);
   W_DstRegAdr:   in std_logic_vector(4 downto 0);
   CE_RegWrite2:  in std_logic;
   CM_RegWrite:   in std_logic;
   CW_RegWrite:   in std_logic;
   CE_MemAluMux2: in std_logic;
   CM_MemAluMux:  in std_logic;
   C_Jump:        in std_logic;
   C_JmpType:     in std_logic;
   C_DBranch:     in std_logic;
   C_JalDet:      in std_logic;
   C_FStall:      out std_logic;
   C_DStall:      out std_logic;
   C_JalHazard:   out std_logic;
   C_BraHazard:   out std_logic
);
end entity HazDet;

------------------------------------------------------------------------------------------

architecture stuff of HazDet is

-- COMPONENTS --

component compare5
port
(
   A,B: in std_logic_vector(4 downto 0);
   Eq:  out std_logic
);
end component compare5;

component dff is
port
(
        D:      in std_logic;
        clk:    in std_logic;
        rst:    in std_logic;
        Q:      out std_logic
);
end component dff;

-- SIGNALS --
signal D_RsDep:      std_logic;
signal D_RtDep:      std_logic;
signal E_LWDet:      std_logic;
signal M_LWDet:      std_logic;

signal D_RsEqDstE:   std_logic;
signal D_RtEqDstE:   std_logic;
signal D_RsEqDstM:   std_logic;
signal D_RtEqDstM:   std_logic;
signal E_DstEq31:    std_logic;
signal M_DstEq31:    std_logic;

signal LWHazard:     std_logic;
signal BraHazard:    std_logic;
signal JrHazard:     std_logic;
signal JalHazard:    std_logic;

signal EDstNotZero:  std_logic;
signal MDstNotZero:  std_logic;
signal WDstNotZero:  std_logic;

signal NClk:         std_logic;
signal FStall:       std_logic;
-- HDL --

begin

---------------------------------------------------------------
-- Received from Espresso hazdet.esp file
D_RsDep <= (not O(5) and O(4) and  not O(3) and O(2) and O(1) and  not O(0) and F(5) and  not F(4) and F(3) and  not F(2) and F(1) and  not F(0))  or  ( not O(5) and  not O(4) and  not O(1) and  not O(0) and  not F(5) and  not F(4) and F(3) and  not F(2) and  not F(1) and  not F(0))  or  ( not O(5) and  not O(4) and  not O(3) and  not O(1) and  not O(0) and F(5) and  not F(4) and  not F(3) and F(2))  or  ( not O(5) and  not O(4) and  not O(3) and  not O(1) and  not O(0) and F(5) and  not F(4) and  not F(3) and  not F(0))  or  (O(5) and  not O(4) and  not O(2) and O(1) and O(0))  or  ( not O(5) and O(4) and  not O(3) and O(2) and  not O(1) and O(0))  or  ( not O(5) and  not O(4) and  not O(3) and O(2) and  not O(1) and  not O(0))  or  ( not O(5) and  not O(4) and O(3) and  not O(0))  or  ( not O(5) and  not O(4) and O(3) and  not O(1));
D_RtDep <= (not O(5) and O(4) and  not O(3) and O(2) and O(1) and  not O(0) and F(5) and  not F(4) and F(3) and  not F(2) and F(1) and  not F(0))  or  (O(5) and  not O(4) and O(3) and  not O(2) and O(1) and O(0))  or  ( not O(5) and  not O(4) and  not O(3) and  not O(1) and  not O(0) and F(5) and  not F(4) and  not F(3) and F(2))  or  ( not O(5) and  not O(4) and  not O(3) and  not O(1) and  not O(0) and F(5) and  not F(4) and  not F(3) and  not F(0))  or  ( not O(5) and O(4) and  not O(3) and O(2) and  not O(1) and O(0))  or  ( not O(5) and  not O(4) and  not O(3) and O(2) and  not O(1) and  not O(0));

-- LW at EX Stage
E_LWDet <= CE_RegWrite2 and (not CE_MemAluMux2);
-- LW at MA Stage
M_LWDet <= CM_RegWrite and (not CM_MemAluMux);

-- Comparators (yeah behavioral is easier)
CMP1:   compare5 port map(E_DstRegAdr, D_RsAdr, D_RsEqDstE);
CMP2:   compare5 port map(E_DstRegAdr, D_RtAdr, D_RtEqDstE);
CMP3:   compare5 port map(M_DstRegAdr, D_RsAdr, D_RsEqDstM);
CMP4:   compare5 port map(M_DstRegAdr, D_RtAdr, D_RtEqDstM);
CMP5:   compare5 port map(E_DstRegAdr, "11111", E_DstEq31);
CMP6:   compare5 port map(M_DstRegAdr, "11111", M_DstEq31);

EDstNotZero <= E_DstRegAdr(4) or E_DstRegAdr(3) or E_DstRegAdr(2) or E_DstRegAdr(1) or E_DstRegAdr(0);
MDstNotZero <= M_DstRegAdr(4) or M_DstRegAdr(3) or M_DstRegAdr(2) or M_DstRegAdr(1) or M_DstRegAdr(0);
WDstNotZero <= W_DstRegAdr(4) or W_DstRegAdr(3) or W_DstRegAdr(2) or W_DstRegAdr(1) or W_DstRegAdr(0);

----------------------------------------------------------------------------------------------------
-- LW HAZARD
-- LW detected at EX and its destination is not $0
-- Instruction at ID has RS or RT dependent on result of LW at EX 
--
-- /Comment/ If dependent on instruction that writes to $0 - no hazard/

LWHazard <= E_LWDet and  EDstNotZero and ((D_RsDep and D_RsEqDstE) or (D_RtDep and D_RtEqDstE));

-- BRANCH HAZARD --
-- Branch detected at ID
-- Branch dependent on Instruction at EX or Branch dependent on LW at MA
-- If dependent on instruction that writes to $0 - no hazard

-- JR HAZARD --
-- Jr detected at ID
-- Jr dependent on instruction writing to D_RS at EX or dependent on LW writing to D_RS at MA
-- If dependent on instruction that writes to $0 - no hazard

BraHazard <= C_DBranch and ((CE_RegWrite2 and EDstNotZero and (D_RsEqDstE or D_RtEqDstE)) or (CM_RegWrite and (not CM_MemAluMux) and MDstNotZero and (D_RsEqDstM or D_RtEqDstM)));

JrHazard <= C_Jump and C_JmpType and ((CE_RegWrite2 and EDstNotZero and D_RsEqDstE) or (CM_RegWrite  and MDstNotZero and (not CM_MemAluMux) and D_RsEqDstM));

-- JAL HAZARD
-- Jal at ID
-- Pending RegWrite at WB
-- Pending RegWrite to $31 at MA or EX
-- If dependent on instruction that writes to $0 - no hazard

JalHazard <= C_JalDet and ((CW_RegWrite and WDstNotZero) or (CM_RegWrite and MDstNotZero and M_DstEq31) or (CE_RegWrite2 and EDstNotZero and E_DstEq31));

-------------

-- Stall FETCH stage if Load, Branch, Jump or Jump-and-allocate hazard happens.
FStall <= LWHazard or BraHazard or JrHazard or JalHazard;
NClk <= not Clk;
FSTALLFF:   dff port map(FStall, NClk, Rst, C_FStall);

-- Stall DECODE stage if Load, Branch, Jump or Jump-and-allocate hazard happens.
C_DStall <= LWHazard or BraHazard or JrHazard or JalHazard;

-- Needed in FETCH stage (to control predictor)
C_BraHazard <= BraHazard;
-- Needed in DECODE stage (to control writing to RFile)
C_JalHazard <= JalHazard;

-------------

end stuff;

------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------

