-- Datapath
-- Author: John Moskal
--  Created: 11-28-04
-- Modified: 04-02-05
--
-- Modifications:
--
------------------------------------------------------------------------------------------

library work;
library ieee;
use ieee.std_logic_1164.all;

entity datapath is
port
(
-- Common Control Signals
   C_Clk:         in std_logic;
   C_Rst:         in std_logic;
   
-- Flush
   C_FDFlush:     in std_logic;
   C_DEFLush:     in std_logic;
   C_EMFlush:     in std_logic;

-- Instuction Fetch Control Signals
   C_Exception:      in std_logic;
   C_Jump:      in std_logic;
   C_DBranch:   in std_logic;

-- Instruction Decode Control Signals
   C_BeqBne:      in std_logic;
   C_JmpType:     in std_logic;
   C_JalDet:      in std_logic;

-- Execute Control Signals
   C_AluOpCode:   in std_logic_vector(2 downto 0);
   C_DstRegMux:   in std_logic;
   C_Op2SrcMux:   in std_logic;

-- Memory Access Control Signals
   C_MemWrite:    in std_logic;

-- Write Back Control Signals
   C_MemAluMux:     in std_logic;
   C_RegWrite:    in std_logic;

-- Other Stuff
   C_EPCSrc:      in std_logic;
   C_EPCWrite:    in std_logic;
   C_Cause:       in std_logic;
   C_CauseWrite:  in std_logic;
   
-- ALU Output  
   E_AluCout:     out std_logic;
   E_AluZero:     out std_logic;

-- Exception Output
   O_EPCAdrOut:   out std_logic_vector(31 downto 0);
   O_CauseOut:    out std_logic;
   
-- Control Output
   CO_Opcode:     out std_logic_vector(5 downto 0);
   CO_Func:       out std_logic_vector(5 downto 0);
   CO_ALUOpcode:  out std_logic_vector(2 downto 0);
   CO_ALUOvf:     out std_logic
);
end entity datapath;

------------------------------------------------------------------------------------------

architecture stuff of datapath is

------------------------------------------------------------------------------------------
-- COMPONENTS --

-- MUXES
component MUX2 is
port
(
        A, B:           in std_logic;
        SL:             in std_logic;
        R:              out std_logic
);
end component MUX2;

component MUX4 is
port
(
        A, B, C, D:     in std_logic;
        SL:             in std_logic_vector(1 downto 0);
        R:              out std_logic
);
end component MUX4;

component mux2_3 is
port
(
   A,B:   in std_logic_vector(2 downto 0);
   SL:    in std_logic;
   R:     out std_logic_vector(2 downto 0)
);
end component mux2_3;


component mux2_5 is
port
(
   A,B:   in std_logic_vector(4 downto 0);
   SL:    in std_logic;
   R:     out std_logic_vector(4 downto 0)
);
end component mux2_5;

component mux2_32 is
port
(
   A,B:   in std_logic_vector(31 downto 0);
   SL:    in std_logic;
   R:     out std_logic_vector(31 downto 0)
);
end component mux2_32;

component mux4_32 is
port
(
   A,B,C,D:   in std_logic_vector(31 downto 0);
   SL:        in std_logic_vector(1 downto 0);
   R:         out std_logic_vector(31 downto 0)
);
end component mux4_32;

-- REGISTERS
component dff is
port
(
        D:      in std_logic;
        clk:    in std_logic;
        rst:    in std_logic;
        Q:      out std_logic
);
end component dff;

component reg3 is
port
(
        input:  in std_logic_vector(2 downto 0);
        stb:    in std_logic;
        rst:    in std_logic;
        output:   out std_logic_vector(2 downto 0)
);
end component reg3;

component reg5 is
port
(
        input:    in std_logic_vector(4 downto 0);
        stb:      in std_logic;
        rst:      in std_logic;
        output:   out std_logic_vector(4 downto 0)
);
end component reg5;

component reg32 is
port
(
        input:    in std_logic_vector(31 downto 0);
        stb:      in std_logic;
        rst:      in std_logic;
        output:   out std_logic_vector(31 downto 0)
);
end component reg32;

-- ALUs

component ALU32CLA is
port
(
   A,B:             in std_logic_vector(31 downto 0);
   OP:              in std_logic_vector(2 downto 0);
   Result:          out std_logic_vector(31 downto 0);
   cout:            out std_logic;
   ovf:             out std_logic;
   zero:            out std_logic
);
end component ALU32CLA;

component CLA32 is
port
(
   A,B:             in std_logic_vector(31 downto 0);
   cin:             in std_logic;
   Result:          out std_logic_vector(31 downto 0)
);
end component CLA32;

component ulpadd32 is
port
(
   A:             in std_logic_vector(31 downto 0);
   cin:             in std_logic;
   Result:          out std_logic_vector(31 downto 0)
);
end component ulpadd32;

-- MEMORY

component meminst512 is
port
(
   adr       : in  std_logic_vector(8 downto 0);
   memdata   : out std_logic_vector(31 downto 0)
);
end component meminst512;

component registerfile is
port
(
   readreg1   : in  std_logic_vector(4 downto 0);
   readreg2   : in  std_logic_vector(4 downto 0);
   writereg  : in  std_logic_vector(4 downto 0);
   writedata : in  std_logic_vector(31 downto 0);
   regwrite  : in  std_logic;
   clk       : in  std_logic;
   rst       : in  std_logic;
   readdata1o : out std_logic_vector(31 downto 0);
   readdata2o : out std_logic_vector(31 downto 0)
);
end component registerfile;

component memdata512 is
  port(clk       : in  std_logic;
       memwrite  : in  std_logic;
       adr       : in  std_logic_vector(8 downto 0);
       writedata : in  std_logic_vector(31 downto 0);
       memdata   : out std_logic_vector(31 downto 0));
end component memdata512;

-- OTHERS
component compare32 is
port
(
   A,B: in std_logic_vector(31 downto 0);
   Eq:  out std_logic
);
end component compare32;

component signex16 is
port
(
   I:   in std_logic_vector(15 downto 0);
   O:   out std_logic_vector(31 downto 0)
);
end component signex16;

component zeroext26 is
port
(
   I:   in std_logic_vector(25 downto 0);
   O:   out std_logic_vector(31 downto 0)
);
end component zeroext26;

component predictor is 
port
(
        stb:    in std_logic;
        rst:    in std_logic;
        R:      in std_logic;
        P:      out std_logic
); 
end component predictor;

component BraDetect is
port
(
   OpCode:   in std_logic_vector(5 downto 0);
   Branch:   out std_logic
);
end component BraDetect;

component FwdUnit is
port
(
   CM_RegWrite,
   CW_RegWrite:         in std_logic;
   CM_MemAluMux,
   CW_MemAluMux:          in std_logic;
   M_DstRegAdr,
   W_DstRegAdr:         in std_logic_vector(4 downto 0);
   D_RsAdr,
   E_RsAdr,
   D_RtAdr,
   E_RtAdr:            in std_logic_vector(4 downto 0);
   C_FwdMux1,
   C_FwdMux2,
   C_CompMux1,
   C_CompMux2:         out std_logic_vector(1 downto 0)
);
end component FwdUnit;

component HazDet is
port
(
    Clk:           in std_logic;
    Rst:           in std_logic;
    O,F:           in std_logic_vector(5 downto 0);   -- Opcode and Function
    D_RsAdr,
    D_RtAdr:       in std_logic_vector(4 downto 0);
    E_DstRegAdr:   in std_logic_vector(4 downto 0);
    M_DstRegAdr:   in std_logic_vector(4 downto 0);
    W_DstRegAdr:   in std_logic_vector(4 downto 0);
    CE_RegWrite2:  in std_logic;
    CM_RegWrite:   in std_logic;
    CW_RegWrite:   in std_logic;
    CE_MemAluMux2: in std_logic;
    CM_MemAluMux:  in std_logic;
    C_Jump:        in std_logic;
    C_JmpType:     in std_logic;
    C_DBranch:     in std_logic;
    C_JalDet:      in std_logic;
    C_FStall:      out std_logic;
    C_DStall:      out std_logic;
    C_JalHazard:   out std_logic;
    C_BraHazard:   out std_logic
);
end component HazDet;

------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
-- SIGNALS --

signal Zeros:                   std_logic_vector(31 downto 0);
signal ExcAdr:                  std_logic_vector(31 downto 0);
signal RAAdr:                   std_logic_vector(4 downto 0);

signal C_NClk:                  std_logic;

-- Datapath Signals ---------
-- Fetch Stage
signal F_PCMux1Out,
       F_PCMux2Out,
       F_PCMux3Out,
       F_PCMux4Out:             std_logic_vector(31 downto 0);
signal F_PCNext:                std_logic_vector(31 downto 0);
signal F_BraAdr:                std_logic_vector(31 downto 0);
signal F_PC:                    std_logic_vector(31 downto 0);
signal F_Instr:                 std_logic_vector(31 downto 0);
signal F_Instr2:                std_logic_vector(31 downto 0);
signal F_SEImm:                 std_logic_vector(31 downto 0);
signal F_BraPred:               std_logic;
signal F_PredStb:               std_logic;
signal F_BraDet:                std_logic;
signal C_PCWrite:               std_logic;
signal C_FStall:                std_logic;
signal C_BraHazard:             std_logic;


-- Decode Stage
signal C_ALUOpCode2:            std_logic_vector(2 downto 0);
signal C_DstRegMux2:            std_logic;
signal C_Op2SrcMux2:            std_logic;
signal C_MemWrite2:             std_logic;
signal C_MemAluMux2:            std_logic;
signal C_RegWrite2:             std_logic;
signal C_DStall:                std_logic;
signal C_JalHazard:             std_logic;
signal C_Jal:                   std_logic;
signal C_OpCode:                std_logic_vector(5 downto 0);
signal C_Func:                  std_logic_vector(5 downto 0);

  -- Forwarding Unit for BEQ and BNE
signal C_CompMux1:              std_logic_vector(1 downto 0);
signal C_CompMux2:              std_logic_vector(1 downto 0);   

signal D_JmpAdr:                std_logic_vector(31 downto 0);
signal D_BraAdr:                std_logic_vector(31 downto 0);
signal D_PCNext:                std_logic_vector(31 downto 0);
signal D_BraPred:               std_logic;

signal D_OpCode:                std_logic_vector(5 downto 0);
signal D_RsAdr,
       D_RtAdr,
       D_RdAdr:                 std_logic_vector(4 downto 0);
signal D_Func:                  std_logic_vector(5 downto 0);
signal D_JmpAdr26:              std_logic_vector(25 downto 0);
signal D_NormJmpAdr:            std_logic_vector(31 downto 0);
signal D_RdAdr2:                 std_logic_vector(4 downto 0);
signal D_Instr:                 std_logic_vector(31 downto 0);
signal D_SEImm:                 std_logic_vector(31 downto 0);
signal D_Rs,
       D_Rt,
       D_Rd:                    std_logic_vector(31 downto 0);
signal D_DMux1Out,
       D_DMux2Out:              std_logic_vector(31 downto 0);
signal D_Compare:               std_logic;
signal D_BraTNT:                std_logic;
signal D_BraMissAdr:            std_logic_vector(31 downto 0);
signal D_BraMiss:               std_logic;
signal D_RegWrite:              std_logic;

-- Execute Stage
signal CE_AluOpCode:            std_logic_vector(2 downto 0);
signal CE_DstRegMux:            std_logic;
signal CE_Op2SrcMux:            std_logic;
signal CE_MemWrite:             std_logic;
signal CE_MemAluMux:              std_logic;
signal CE_RegWrite:             std_logic;
signal CE_MemWrite2:            std_logic;
signal CE_MemAluMux2:             std_logic;
signal CE_RegWrite2:            std_logic;
   -- Forwarding Unit
signal C_FwdMux1:               std_logic_vector(1 downto 0);
signal C_FwdMux2:               std_logic_vector(1 downto 0); 

signal E_PCNext:                std_logic_vector(31 downto 0);
signal E_Rs,
       E_Rt:                    std_logic_vector(31 downto 0);
signal E_SEImm:                 std_logic_vector(31 downto 0);
signal E_RsAdr,
       E_RtAdr,
       E_RdAdr:                 std_logic_vector(4 downto 0);
signal E_RtFwd:                 std_logic_vector(31 downto 0);
signal E_Op1,
       E_Op2:                   std_logic_vector(31 downto 0);
signal E_AluRslt:               std_logic_vector(31 downto 0);
signal E_DstRegAdr:             std_logic_vector(4 downto 0);


-- Memory Access Stage
signal CM_MemWrite:             std_logic;
signal CM_MemAluMux:            std_logic;
signal CM_RegWrite:             std_logic;

signal M_AluRslt:               std_logic_vector(31 downto 0);
signal M_Fwd:                   std_logic_vector(31 downto 0);
signal M_MDataIn:               std_logic_vector(31 downto 0);
signal M_MDataOut:              std_logic_vector(31 downto 0);
signal M_DstRegAdr:             std_logic_vector(4 downto 0);

-- Write Back Stage
signal CW_MemAluMux:            std_logic;
signal CW_RegWrite:             std_logic;

signal W_MDataOut:              std_logic_vector(31 downto 0);
signal W_AluRslt:               std_logic_vector(31 downto 0);
signal W_Fwd:                   std_logic_vector(31 downto 0);
signal W_DstRegAdr:             std_logic_vector(4 downto 0);

-- Control Signals generated inside Datapath ---------
signal C_FStallClk:             std_logic;
signal C_FBranch:               std_logic;
signal C_BraMiss:               std_logic;

signal C_FDFlush2:              std_logic;

signal C_DECtrlSrc:             std_logic;

-- Other stuff

signal O_EPCAdrIn:              std_logic_vector(31 downto 0);
signal C_EPCClk:                std_logic;
signal C_CauseClk:              std_logic;

------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
-- HDL --

begin

Zeros  <= "00000000000000000000000000000000";
ExcAdr <= "00000000000000000000000000000000";
RAAdr  <= "11111";

------------------

-- Fetch Stage
PCMUX1:     mux2_32    port map(F_PCMux2Out, F_BraAdr, C_FBranch, F_PCMux1Out );
PCMUX2:     mux2_32    port map(F_PCMux3Out, ExcAdr, C_Exception, F_PCMux2Out);
PCMUX3:     mux2_32    port map(F_PCMux4Out, D_JmpAdr, C_Jump, F_PCMux3Out);
PCMUX4:     mux2_32    port map(F_PCNext, D_BraMissAdr, C_BraMiss, F_PCMux4Out);
PCMUX5:     mux2_32    port map(D_BraAdr, D_PCNext, D_BraPred, D_BraMissAdr);
PC:         reg32      port map(F_PCMux1Out, C_FStallClk, C_Rst, F_PC);
ULPADD:     ulpadd32   port map(F_PC, '1', F_PCNext);
INSTMEM:    meminst512 port map(F_PC(8 downto 0), F_Instr);
SIGNEX:     signex16   port map(F_Instr(15 downto 0), F_SEImm);
BALU:       CLA32      port map(F_SEImm, F_PCNext,'0',F_BraAdr);
PRED:       predictor  port map(F_PredStb, C_Rst, D_BraTNT, F_BraPred);
BDETECT:    BraDetect  port map(F_Instr(31 downto 26), F_BraDet);

FLUSHMUX:   mux2_32    port map(F_Instr, Zeros, C_FDFlush2, F_Instr2);
C_FDFlush2 <= (C_FDFlush or C_BraMiss or C_Jump);

F_PredStb <= (not C_Clk) and (not C_BraHazard) and C_DBranch;
C_FBranch <= F_BraPred and F_BraDet;

C_FStallClk <= C_Clk and (not C_FStall);

-- Decode Stage
FD_BranchAdr:  reg32 port map(F_BraAdr,  C_FStallClk, C_Rst, D_BraAdr);
FD_PCNext:     reg32 port map(F_PCNext,  C_FStallClk, C_Rst, D_PCNext);
FD_Inst:       reg32 port map(F_Instr,   C_FStallClk, C_Rst, D_Instr);
FD_SEImm:      reg32 port map(F_SEImm,   C_FStallClk, C_Rst, D_SEImm);
FD_BraPred:    dff   port map(F_BraPred, C_FStallClk, C_Rst, D_BraPred);

DEMxE1:   mux2_3 port map(C_ALUOpCode, "000", C_DECtrlSrc, C_ALUOpCode2);
DEMxE2:   mux2   port map(C_DstRegMux, '0',   C_DECtrlSrc, C_DstRegMux2);
DEMxE3:   mux2   port map(C_Op2SrcMux, '0',   C_DECtrlSrc, C_Op2SrcMux2);
DEMxM1:   mux2   port map(C_MemWrite,  '0',   C_DECtrlSrc, C_MemWrite2);
DEMxW1:   mux2   port map(C_MemAluMux, '0',   C_DECtrlSrc, C_MemAluMux2);
DEMxW2:   mux2   port map(C_RegWrite,  '0',   C_DECtrlSrc, C_RegWrite2);

C_Opcode    <= D_Instr(31 downto 26);
CO_Opcode    <= C_Opcode;
C_Func      <= D_Instr(05 downto 00);
CO_Func      <= C_Func;

D_RsAdr     <= D_Instr(25 downto 21);
D_RtAdr     <= D_Instr(20 downto 16);
D_RdAdr     <= D_Instr(15 downto 11);
D_JmpAdr26  <= D_Instr(25 downto 00);

C_Jal <= C_JalDet and (not C_JalHazard);
D_RegWrite <= C_Jal or CW_RegWrite;

ZEROEX:    zeroext26    port map(D_JmpAdr26, D_NormJmpAdr);
RADR3MUX:  mux2_5       port map(W_DstRegAdr, RAAdr, C_Jal, D_RdAdr2);
RDAT3MUX:  mux2_32      port map(W_Fwd, D_PCNext, C_Jal, D_Rd);
REGFIL:    registerfile port map(D_RsAdr, D_RtAdr, D_RdAdr2, D_Rd, D_RegWrite, C_Clk, C_Rst, D_Rs, D_Rt);
DMUX1:     mux4_32      port map(D_Rs, M_Fwd, W_Fwd, Zeros, C_CompMux1, D_DMux1Out);
DMUX2:     mux4_32      port map(D_Rt, M_Fwd, W_Fwd, Zeros, C_CompMux2, D_DMux2Out);
COMP:      compare32    port map(D_DMux1Out, D_DMux2Out, D_Compare);
JMPMUX:    mux2_32      port map(D_NormJmpAdr, D_DMux1Out, C_JmpType, D_JmpAdr);

D_BraTNT <= C_BeqBne xor D_Compare;
D_BraMiss <= D_BraPred xor D_BraTNT;
C_BraMiss <= D_BraMiss and C_DBranch;
C_DECtrlSrc <= C_DStall or C_DEFlush;

-- Execute Stage
DE_E1:      reg3 port map(C_AluOpCode2, C_Clk, C_Rst, CE_AluOpCode);
DE_E2:      dff port map(C_DstRegMux2, C_Clk, C_Rst, CE_DstRegMux);
DE_E3:      dff port map(C_Op2SrcMux2, C_Clk, C_Rst, CE_Op2SrcMux);
DE_M1:      dff port map(C_MemWrite2, C_Clk, C_Rst, CE_MemWrite);
DE_W1:      dff port map(C_MemAluMux2, C_Clk, C_Rst, CE_MemAluMux);
DE_W2:      dff port map(C_RegWrite2, C_Clk, C_Rst, CE_RegWrite);

EMMxM1:   mux2 port map(CE_MemWrite, '0', C_EMFlush, CE_MemWrite2);
EMMxW1:   mux2 port map(CE_MemAluMux, '0', C_EMFlush, CE_MemAluMux2);
EMMxW2:   mux2 port map(CE_RegWrite, '0', C_EMFlush, CE_RegWrite2);

DE_PCNext:   reg32 port map(D_PCNext, C_Clk, C_Rst, E_PCNext);
DE_RData1:   reg32 port map(D_Rs, C_Clk, C_Rst, E_Rs);
DE_RData2:   reg32 port map(D_Rt, C_Clk, C_Rst, E_Rt);
DE_SEImm:    reg32 port map(D_SEImm, C_Clk, C_Rst, E_SEImm);
DE_Rs:       reg5  port map(D_RsAdr, C_Clk, C_Rst, E_RsAdr);
DE_Rt:       reg5  port map(D_RtAdr, C_Clk, C_Rst, E_RtAdr);
DE_Rd:       reg5  port map(D_RdAdr, C_Clk, C_Rst, E_RdAdr);

FWDMUX1:   mux4_32   port map(E_Rs, M_Fwd, W_Fwd, Zeros, C_FwdMux1, E_Op1);
FWDMUX2:   mux4_32   port map(E_Rt, M_Fwd, W_Fwd, Zeros, C_FwdMux2, E_RtFwd);
OP2MUX:   mux2_32 port map(E_RtFwd, E_SEImm, CE_Op2SrcMux, E_Op2);
ALU:      ALU32CLA port map(E_Op1, E_Op2, CE_AluOpCode, E_AluRslt, E_AluCout, CO_AluOvf, E_AluZero);
DSTMUX:   mux2_5   port map(E_RtAdr, E_RdAdr, CE_DstRegMux, E_DstRegAdr);

CO_ALUOpcode <= CE_ALUOpcode;

-- Memory Access Stage
EM_W1:      dff port map(CE_MemAluMux2, C_Clk, C_Rst, CM_MemAluMux);
EM_W2:      dff port map(CE_RegWrite2, C_Clk, C_Rst, CM_RegWrite);
EM_M1:      dff port map(CE_MemWrite2, C_Clk, C_Rst, CM_MemWrite);

EM_ALURes:   reg32 port map(E_AluRslt, C_Clk, C_Rst, M_AluRslt);
EM_RData2:   reg32 port map(E_RtFwd, C_Clk, C_Rst, M_MDataIn);
EM_DstReg:   reg5  port map(E_DstRegAdr, C_Clk, C_Rst, M_DstRegAdr);

M_Fwd <= M_AluRslt;

DATMEM:   memdata512 port map(C_Clk, CM_MemWrite, M_AluRslt(8 downto 0), M_MDataIn, M_MDataOut);

--MFWDMUX:   mux2_32 port map(M_MData, M_AluRslt, CM_MemAluMux, M_Fwd);

-- Write Back Stage
MW_W1:      dff port map(CM_MemAluMux, C_Clk, C_Rst, CW_MemAluMux);
MW_W2:      dff port map(CM_RegWrite, C_Clk, C_Rst, CW_RegWrite);

MW_Data:   reg32 port map(M_MDataOut, C_Clk, C_Rst, W_MDataOut);
MW_ALURes: reg32 port map(M_AluRslt, C_Clk, C_Rst, W_AluRslt);
MW_DstReg: reg5  port map(M_DstRegAdr, C_Clk, C_Rst, W_DstRegAdr);

DATMUX:   mux2_32 port map(W_MDataOut, W_AluRslt, CW_MemAluMux, W_Fwd);

-- OTHER Stuff
EPCMUX:   mux2_32 port map(D_PCNext, E_PCNext, C_EPCSrc, O_EPCAdrIn);
EPC:      reg32 port map(O_EPCAdrIn, C_EPCClk, C_Rst, O_EPCAdrOut);
C_EPCClk <= C_EPCWrite and not(C_Clk);

CAUSE:   dff port map(C_Cause, C_CauseClk, C_Rst, O_CauseOut);
C_CauseClk <= C_CauseWrite and not(C_Clk);

-- Hazard Detection and Forwarding
FDU:   FwdUnit port map(CM_RegWrite,CW_RegWrite,CM_MemAluMux,CW_MemAluMux,M_DstRegAdr,W_DstRegAdr,D_RsAdr,E_RsAdr,D_RtAdr,E_RtAdr,C_FwdMux1,C_FwdMux2,C_CompMux1,C_CompMux2);
HDU:   HazDet  port map(C_Clk,C_Rst,C_Opcode,C_Func,D_RsAdr,D_RtAdr,E_DstRegAdr,M_DstRegAdr,W_DstRegAdr,CE_RegWrite2,CM_RegWrite,CW_RegWrite,CE_MemAluMux2,CM_MemAluMux,C_Jump,C_JmpType,C_DBranch,C_JalDet,C_FStall,C_DStall,C_JalHazard,C_BraHazard);

end stuff;


------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------

