-- Simple Control for Datapth
-- Author: John Moskal
--  Created: 12-05-04
-- Modified: 12-05-04
--
-- Modifications:
--
------------------------------------------------------------------------------------------

library work;
library ieee;
use ieee.std_logic_1164.all;

entity simplecontrol is
port
(
   rst:         in std_logic;
   opcode:       in std_logic_vector(5 downto 0);
   func:         in std_logic_vector(5 downto 0);
   ALUOpcode: in std_logic_vector(2 downto 0);
   ALUOvf:     in std_logic;
   
   C_FDFlush:    out std_logic;
   C_DEFlush:    out std_logic;
   C_EMFlush:    out std_logic;
   C_Exception:  out std_logic;
   C_Jump:       out std_logic;
   C_DBranch:    out std_logic;
   C_BeqBne:     out std_logic;
   C_JmpType:    out std_logic;
   C_JalDet:     out std_logic;
   C_ALUOpcode:  out std_logic_vector(2 downto 0);
   C_DstRegMux:  out std_logic;
   C_Op2SrcMux:  out std_logic;
   C_MemWrite:   out std_logic;
   C_MemAluMux:    out std_logic;
   C_RegWrite:   out std_logic;
   C_EPCSrc:     out std_logic;
   C_EPCWrite:   out std_logic;
   C_CauseWrite: out std_logic;
   C_Cause:      out std_logic
);
end entity simplecontrol;

------------------------------------------------------------------------------------------

architecture stuff of simplecontrol is

-- COMPONENTS --

-- SIGNALS --

-- HDL --

begin
process(opcode, func, ALUOpcode, ALUOvf)
begin

if (rst='1') then 
-- NOP
          -- NOP
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '0';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '0';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
else
   if (((ALUOpcode="100") or (ALUOpcode="101")) and ALUOvf='1') then
      -- Overflow Exception
          C_FDFlush    <= '1';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '1';    -- Decode/Execute Register Flush
          C_EMFlush    <= '1';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '0';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '0';    -- Register Write Enable
          C_Exception  <= '1';    -- Exception
          C_EPCSrc     <= '1';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '1';    -- EPC Write Enable
          C_CauseWrite <= '1';    -- Cause Write Enable
          C_Cause      <= '1';    -- Exception Cause        
   else
       if ((opcode="000000") and (func="000000")) then
          -- NOP
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '0';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '0';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif ((opcode="000000") and (func="100100")) then
          -- AND
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "010";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '1';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif ((opcode="000000") and (func="100101")) then
          -- OR
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '1';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif ((opcode="000000") and (func="100110")) then
          -- XOR
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "011";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '1';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif ((opcode="000000") and (func="100111")) then
          -- NOR
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "001";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '1';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif ((opcode="000000") and (func="100000")) then
          -- ADD
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "100";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '1';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif ((opcode="000000") and (func="100010")) then
          -- SUB
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "101";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '1';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif ((opcode="000000") and (func="001000")) then
          -- JR
          C_FDFlush    <= '1';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '1';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '1';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '0';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '0';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif ((opcode="010110") and (func="101010")) then
          -- SLT
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "111";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '1';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif (opcode="001100") then
          -- ANDI
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "010";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '1';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif (opcode="001101") then
          -- ORI
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '1';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause          
       elsif (opcode="001110") then
          -- XORI
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "011";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '1';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif (opcode="001000") then
          -- ADDI
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "100";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '1';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif (opcode="001001") then
          -- SUBI
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "101";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '1';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif (opcode="001010") then
          -- SLTI
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "111";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '1';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '1';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif (opcode="100011") then
          -- LW
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '1';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '0';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '1';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif (opcode="101011") then
          -- SW
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '1';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '1';    -- Memory Write Enable
          C_MemAluMux    <= '0';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '0';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif (opcode="000100") then
          -- BEQ
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '1';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '0';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '0';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif (opcode="010101") then
          -- BNE
          C_FDFlush    <= '0';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '1';    -- Branch in Decode Stage
          C_BeqBne     <= '1';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '0';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '0';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif (opcode="000010") then
          -- J
          C_FDFlush    <= '1';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '1';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '0';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '0';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       elsif (opcode="000011") then
          -- JAL
          C_FDFlush    <= '1';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '0';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '1';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '1';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '0';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '0';    -- Register Write Enable
          C_Exception  <= '0';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '0';    -- EPC Write Enable
          C_CauseWrite <= '0';    -- Cause Write Enable
          C_Cause      <= '0';    -- Exception Cause            
       else
          -- INVALID
          C_FDFlush    <= '1';    -- Fetch/Decode Register Flush
          C_DEFlush    <= '1';    -- Decode/Execute Register Flush
          C_EMFlush    <= '0';    -- Execute/MemoryAccess Register Flush
          C_Jump       <= '0';    -- Jump
          C_DBranch    <= '0';    -- Branch in Decode Stage
          C_BeqBne     <= '0';    -- Beq (0) / Bne (1)
          C_JmpType    <= '0';    -- Jump Type - Normal Jump (0) / Register Jump (1)
          C_JalDet     <= '0';    -- JAL in Decode Stage
          C_ALUOpcode  <= "000";  -- ALU Opcode (see ALU32CLA.vhd)
          C_DstRegMux  <= '0';    -- Destination Register Address Mux - RT (0) / RD (1)
          C_Op2SrcMux  <= '0';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
          C_MemWrite   <= '0';    -- Memory Write Enable
          C_MemAluMux    <= '0';    -- Data Mux - Memory Data (0) / ALU Result (1)
          C_RegWrite   <= '0';    -- Register Write Enable
          C_Exception  <= '1';    -- Exception
          C_EPCSrc     <= '0';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
          C_EPCWrite   <= '1';    -- EPC Write Enable
          C_CauseWrite <= '1';    -- Cause Write Enable
          C_Cause      <= '1';    -- Exception Cause            
       end if;
   end if;
end if;
end process;
end stuff;

------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
 
--          C_FDFlush    <= '';    -- Fetch/Decode Register Flush
--          C_DEFlush    <= '';    -- Decode/Execute Register Flush
--          C_EMFlush    <= '';    -- Execute/MemoryAccess Register Flush
--          C_Jump       <= '';    -- Jump
--          C_DBranch    <= '';    -- Branch in Decode Stage
--          C_BeqBne     <= '';    -- Beq (0) / Bne (1)
--          C_JmpType    <= '';    -- Jump Type - Normal Jump (0) / Register Jump (1)
--          C_JalDet     <= '';    -- JAL in Decode Stage
--          C_ALUOpcode  <= "";  -- ALU Opcode (see ALU32CLA.vhd)
--          C_DstRegMux  <= '';    -- Destination Register Address Mux - RT (0) / RD (1)
--          C_Op2SrcMux  <= '';    -- ALU Operand Source - Register Data 2 (0) / Sign Extended Immediate (1)
--          C_MemWrite   <= '';    -- Memory Write Enable
--          C_MemAluMux    <= '';    -- Data Mux - Memory Data (0) / ALU Result (1)
--          C_RegWrite   <= '';    -- Register Write Enable
--          C_Exception  <= '';    -- Exception
--          C_EPCSrc     <= '';    -- EPC Source - D_PCNext (0) / E_PCNext (1)
--          C_EPCWrite   <= '';    -- EPC Write Enable
--          C_CauseWrite <= '';    -- Cause Write Enable
--          C_Cause      <= '';    -- Exception Cause


