-------------------------------------------------------------------------------
-- File name : bit_alu.vhdl
-- Title     : 1-bit ALU
-- project   : ECE 485
-- Library   : single-cycle processor
-- Author(s) : James E. Stine, Jr.
-- Purpose   : definition of single-cycle processor
-- notes :   

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

entity bit_alu is
  port(a     : in std_logic;
       b     : in std_logic;
       c_in  : in std_logic;
       less  : in std_logic;
       op    : in std_logic_vector(2 downto 0);
       x     : out std_logic;
       c_out : out std_logic);
end bit_alu;

architecture structure OF bit_alu is

  component mux41
    port (D0 : in  std_logic;
          D1 : in  std_logic;
          D2 : in  std_logic;
          D3 : in  std_logic;
          S  : in  std_logic_vector(1 downto 0);
          Y  : out std_logic);
  end component;

  signal sum : std_logic;
  signal bxored : std_logic;
  signal node_and, node_or, node_xor, node1, node2: std_logic;

begin
  
  bxored <= (b xor op(2));  
  node_and <= (a and bxored);				
  node_or <= (a or bxored);				
  node_xor <= ((not node_and) and node_or);		
  node1 <= (c_in and node_xor);
  node2 <= (node_xor or c_in);
  sum <= ((not node1) and node2);			  
  c_out <= (node_and or node1);
  
  MX1: mux41 port map (node_and, node_or, sum, less, op(1 downto 0), x);
  
end structure;

